@@ -1,13 +1,14 @@
 #This file is part of Tryton.  The COPYRIGHT file at the top level of
 #this repository contains the full copyright notices and license terms.
 "Sales extension for managing leads and opportunities"
-from __future__ import with_statement
 import datetime
-from trytond.model import ModelView, ModelSQL, ModelWorkflow, fields
-from trytond.wizard import Wizard
+import time
+from trytond.model import ModelView, ModelSQL, Workflow, fields
+from trytond.wizard import Wizard, StateView, StateAction, Button
 from trytond.backend import FIELDS
 from trytond.pyson import Equal, Eval, Not, In, If, Get, PYSONEncoder
 from trytond.transaction import Transaction
+from trytond.pool import Pool
 
 STATES = [
     ('lead', 'Lead'),
@@ -17,77 +18,75 @@ STATES = [
     ('lost', 'Lost'),
 ]
 _STATES_START = {
-    'readonly': Not(Equal(Eval('state'), 'lead')),
-}
+    'readonly': Eval('state') != 'lead',
+    }
+_DEPENDS_START = ['state']
 _STATES_STOP = {
     'readonly': In(Eval('state'), ['converted', 'lost', 'cancelled']),
 }
+_DEPENDS_STOP = ['state']
 
 
-class SaleOpportunity(ModelWorkflow, ModelSQL, ModelView):
+class SaleOpportunity(Workflow, ModelSQL, ModelView):
     'Sale Opportunity'
     _name = "sale.opportunity"
     _description = __doc__
     _history = True
     _rec_name = 'description'
 
-    party = fields.Many2One('party.party', 'Party', required=True, select=1,
-            states=_STATES_STOP, depends=['state'])
+    party = fields.Many2One('party.party', 'Party', required=True, select=True,
+            states=_STATES_STOP, depends=_DEPENDS_STOP)
     address = fields.Many2One('party.address', 'Address',
-            domain=[('party', '=', Eval('party'))],
-            select=2, depends=['party', 'state'],
-            states=_STATES_STOP)
+        domain=[('party', '=', Eval('party'))],
+        select=True, depends=['party', 'state'],
+        states=_STATES_STOP)
     company = fields.Many2One('company.company', 'Company', required=True,
-            select=1, states=_STATES_STOP, domain=[
-                ('id', If(In('company', Eval('context', {})), '=', '!='),
-                    Get(Eval('context', {}), 'company', 0)),
-            ], on_change=['company'], depends=['state'])
+        select=True, states=_STATES_STOP, domain=[
+            ('id', If(In('company', Eval('context', {})), '=', '!='),
+                Get(Eval('context', {}), 'company', 0)),
+            ], on_change=['company'], depends=_DEPENDS_STOP)
     currency = fields.Function(fields.Many2One('currency.currency',
         'Currency'), 'get_currency')
     currency_digits = fields.Function(fields.Integer('Currency Digits'),
             'get_currency_digits')
     amount = fields.Numeric('Amount', digits=(16, Eval('currency_digits', 2)),
             depends=['currency_digits'], help='Estimated revenue amount')
-    warehouse = fields.Many2One('stock.location', 'Warehouse',
-            domain=[('type', '=', 'warehouse')], states={
-                'required': Equal(Eval('state'), 'converted'),
-                'readonly': In(Eval('state'),
-                    ['converted', 'lost', 'cancelled']),
-            })
     payment_term = fields.Many2One('account.invoice.payment_term',
-            'Payment Term', states={
-                'required': Equal(Eval('state'), 'converted'),
-                'readonly': In(Eval('state'),
-                    ['converted', 'lost', 'cancelled']),
-            })
+        'Payment Term', states={
+            'required': Eval('state') == 'converted',
+            'readonly': In(Eval('state'),
+                ['converted', 'lost', 'cancelled']),
+            },
+        depends=['state'])
     employee = fields.Many2One('company.employee', 'Employee', required=True,
             states=_STATES_STOP, depends=['state', 'company'],
             domain=[('company', '=', Eval('company'))])
-    start_date = fields.Date('Start Date', required=True, select=1,
-            states=_STATES_START, depends=['state'])
-    end_date = fields.Date('End Date', select=2, readonly=True, states={
+    start_date = fields.Date('Start Date', required=True, select=True,
+        states=_STATES_START, depends=_DEPENDS_START)
+    end_date = fields.Date('End Date', select=True, readonly=True, states={
         'invisible': Not(In(Eval('state'),
             ['converted', 'cancelled', 'lost'])),
     }, depends=['state'])
     description = fields.Char('Description', required=True,
-            states=_STATES_STOP, depends=['state'])
-    comment = fields.Text('Comment', states=_STATES_STOP)
+        states=_STATES_STOP, depends=_DEPENDS_STOP)
+    comment = fields.Text('Comment', states=_STATES_STOP,
+        depends=_DEPENDS_STOP)
     lines = fields.One2Many('sale.opportunity.line', 'opportunity', 'Lines',
-            states=_STATES_STOP, depends=['state'])
-    state = fields.Selection(STATES, 'State', required=True, select=1,
+        states=_STATES_STOP, depends=_DEPENDS_STOP)
+    state = fields.Selection(STATES, 'State', required=True, select=True,
             sort=False, readonly=True)
-    probability = fields.Integer('Conversion Probability',
+    probability = fields.Integer('Conversion Probability', required=True,
             states={
                 'readonly': Not(In(Eval('state'), ['opportunity', 'lead'])),
             }, depends=['state'], help="Percentage between 0 and 100")
     history = fields.One2Many('sale.opportunity.history', 'opportunity',
             'History', readonly=True)
     lost_reason = fields.Text('Reason for loss', states={
-        'invisible': Not(Equal(Eval('state'), 'lost')),
-    }, depends=['state'])
+            'invisible': Eval('state') != 'lost',
+            }, depends=['state'])
     sale = fields.Many2One('sale.sale', 'Sale', readonly=True, states={
-        'invisible': Not(Equal(Eval('state'), 'converted')),
-        }, depends=['state'])
+            'invisible': Eval('state') != 'converted',
+            }, depends=['state'])
 
     def __init__(self):
         super(SaleOpportunity, self).__init__()
@@ -97,49 +96,70 @@ class SaleOpportunity(ModelWorkflow, ModelSQL, ModelView):
                 'CHECK(probability >= 0 AND probability <= 100)',
                 'Probability must be between 0 and 100!')
         ]
-        self._rpc.update({
-            'button_lead': True,
-        })
+        self._error_messages.update({
+                'delete_cancel': 'Sale Opportunity "%s" must be cancelled '\
+                    'before deletion!',
+                })
+        self._transitions |= set((
+                ('lead', 'opportunity'),
+                ('lead', 'lost'),
+                ('lead', 'cancelled'),
+                ('opportunity', 'converted'),
+                ('opportunity', 'lead'),
+                ('opportunity', 'lost'),
+                ('opportunity', 'cancelled'),
+                ('lost', 'lead'),
+                ('cancelled', 'lead'),
+                ))
+        self._buttons.update({
+                'lead': {
+                    'invisible': ~Eval('state').in_(
+                        ['cancelled', 'lost', 'opportunity']),
+                    'icon': If(Eval('state').in_(['cancelled', 'lost']),
+                        'tryton-clear', 'tryton-go-previous'),
+                    },
+                'opportunity': {
+                    'invisible': ~Eval('state').in_(['lead']),
+                    },
+                'convert': {
+                    'invisible': ~Eval('state').in_(['opportunity']),
+                    },
+                'lost': {
+                    'invisible': ~Eval('state').in_(['lead', 'opportunity']),
+                    },
+                'cancel': {
+                    'invisible': ~Eval('state').in_(['lead', 'opportunity']),
+                    },
+                })
 
     def default_state(self):
         return 'lead'
 
     def default_start_date(self):
-        date_obj = self.pool.get('ir.date')
+        date_obj = Pool().get('ir.date')
         return date_obj.today()
 
     def default_probability(self):
         return 50
 
     def default_company(self):
-        return Transaction().context.get('company') or False
+        return Transaction().context.get('company')
 
     def default_employee(self):
-        user_obj = self.pool.get('res.user')
-        employee_obj = self.pool.get('company.employee')
+        user_obj = Pool().get('res.user')
 
-        employee_id = False
         if Transaction().context.get('employee'):
-            employee_id = Transaction().context['employee']
+            return Transaction().context['employee']
         else:
             user = user_obj.browse(Transaction().user)
             if user.employee:
-                employee_id = user.employee.id
-        return employee_id
+                return user.employee.id
 
     def default_payment_term(self):
-        payment_term_obj = self.pool.get('account.invoice.payment_term')
+        payment_term_obj = Pool().get('account.invoice.payment_term')
         payment_term_ids = payment_term_obj.search(self.payment_term.domain)
         if len(payment_term_ids) == 1:
             return payment_term_ids[0]
-        return False
-
-    def default_warehouse(self):
-        location_obj = self.pool.get('stock.location')
-        location_ids = location_obj.search(self.warehouse.domain)
-        if len(location_ids) == 1:
-            return location_ids[0]
-        return False
 
     def get_currency(self, ids, name):
         res = {}
@@ -154,7 +174,7 @@ class SaleOpportunity(ModelWorkflow, ModelSQL, ModelView):
         return res
 
     def on_change_company(self, values):
-        company_obj = self.pool.get('company.company')
+        company_obj = Pool().get('company.company')
 
         res = {}
         if values.get('company'):
@@ -165,16 +185,16 @@ class SaleOpportunity(ModelWorkflow, ModelSQL, ModelView):
         return res
 
     def on_change_party(self, values):
-        party_obj = self.pool.get('party.party')
-        payment_term_obj = self.pool.get('account.invoice.payment_term')
+        party_obj = Pool().get('party.party')
+        payment_term_obj = Pool().get('account.invoice.payment_term')
 
         res = {
-            'payment_term': False,
+            'payment_term': None,
         }
-        if vals.get('party'):
-            party = party_obj.browse(vals['party'])
-            if party.payment_term:
-                res['payment_term'] = party.payment_term.id
+        if values.get('party'):
+            party = party_obj.browse(values['party'])
+            if party.customer_payment_term:
+                res['payment_term'] = party.customer_payment_term.id
         if not res['payment_term']:
             res['payment_term'] = self.default_payment_term()
         if res['payment_term']:
@@ -182,23 +202,6 @@ class SaleOpportunity(ModelWorkflow, ModelSQL, ModelView):
                     res['payment_term']).rec_name
         return res
 
-    def set_end_date(self, opportunity_id, state):
-        """
-        This will fill the end_date for a lead/opportunity and change the state
-
-        :param opportunity_id: the id of the opportunity
-        :param state: the target state
-        """
-        date_obj = self.pool.get('ir.date')
-        self.write(opportunity_id, {
-            'end_date': date_obj.today(),
-            'state': state,
-            })
-
-    def button_lead(self, ids):
-        self.workflow_trigger_create(ids)
-        return True
-
     def _get_sale_line_opportunity_line(self, opportunity):
         '''
         Return sale line values for each opportunity line
@@ -208,9 +211,11 @@ class SaleOpportunity(ModelWorkflow, ModelSQL, ModelView):
         :return: a dictionary with opportunity line id as key
             and a dictionary of sale line values as value
         '''
-        line_obj = self.pool.get('sale.opportunity.line')
+        line_obj = Pool().get('sale.opportunity.line')
         res = {}
         for line in opportunity.lines:
+            if line.sale_line:
+                continue
             val = line_obj.get_sale_line(line)
             if val:
                 res[line.id] = val
@@ -229,7 +234,6 @@ class SaleOpportunity(ModelWorkflow, ModelSQL, ModelView):
             'party': opportunity.party.id,
             'payment_term': opportunity.payment_term.id,
             'company': opportunity.company.id,
-            'warehouse': opportunity.warehouse.id,
             'invoice_address': opportunity.address and opportunity.address.id,
             'shipment_address': opportunity.address and opportunity.address.id,
             'currency': opportunity.company.currency.id,
@@ -237,17 +241,17 @@ class SaleOpportunity(ModelWorkflow, ModelSQL, ModelView):
         }
         return res
 
-    def create_sale(self, opportunity_id):
+    def create_sale(self, opportunity):
         '''
-        Create a sale for the opportunity
-
-        :param opportunity_id: the id of the opportunity
+        Create a sale for the opportunity and return the sale id
         '''
-        sale_obj = self.pool.get('sale.sale')
-        sale_line_obj = self.pool.get('sale.line')
-        line_obj = self.pool.get('sale.opportunity.line')
+        pool = Pool()
+        sale_obj = pool.get('sale.sale')
+        sale_line_obj = pool.get('sale.line')
+        line_obj = pool.get('sale.opportunity.line')
 
-        opportunity = self.browse(opportunity_id)
+        if opportunity.sale:
+            return
 
         values = self._get_sale_opportunity(opportunity)
         sale_line_values = self._get_sale_line_opportunity_line(opportunity)
@@ -263,11 +267,57 @@ class SaleOpportunity(ModelWorkflow, ModelSQL, ModelView):
                 'sale_line': sale_line_id,
                 })
 
-        self.write(opportunity_id, {
+        self.write(opportunity.id, {
             'sale': sale_id,
             })
         return sale_id
 
+    def delete(self, ids):
+        if isinstance(ids, (int, long)):
+            ids = [ids]
+        # Cancel before delete
+        self.cancel(ids)
+        for opportunity in self.browse(ids):
+            if opportunity.state != 'cancel':
+                self.raise_user_error('delete_cancel', opportunity.rec_name)
+        return super(SaleOpportunity, self).delete(ids)
+
+    @ModelView.button
+    @Workflow.transition('lead')
+    def lead(self, ids):
+        pass
+
+    @ModelView.button
+    @Workflow.transition('opportunity')
+    def opportunity(self, ids):
+        pass
+
+    @ModelView.button
+    @Workflow.transition('converted')
+    def convert(self, ids):
+        date_obj = Pool().get('ir.date')
+        self.write(ids, {
+                'end_date': date_obj.today(),
+                })
+        for opportunity in self.browse(ids):
+            self.create_sale(opportunity)
+
+    @ModelView.button
+    @Workflow.transition('lost')
+    def lost(self, ids):
+        date_obj = Pool().get('ir.date')
+        self.write(ids, {
+                'end_date': date_obj.today(),
+                })
+
+    @ModelView.button
+    @Workflow.transition('cancelled')
+    def cancel(self, ids):
+        date_obj = Pool().get('ir.date')
+        self.write(ids, {
+                'end_date': date_obj.today(),
+                })
+
 SaleOpportunity()
 
 
@@ -279,7 +329,7 @@ class SaleOpportunityLine(ModelSQL, ModelView):
     _history = True
 
     opportunity = fields.Many2One('sale.opportunity', 'Opportunity')
-    sequence = fields.Integer('Sequence')
+    sequence = fields.Integer('Sequence', required=True)
     product = fields.Many2One('product.product', 'Product', required=True,
             domain=[('salable', '=', True)], on_change=['product', 'unit'])
     quantity = fields.Float('Quantity', required=True,
@@ -288,9 +338,9 @@ class SaleOpportunityLine(ModelSQL, ModelView):
     unit_digits = fields.Function(fields.Integer('Unit Digits',
         on_change_with=['unit']), 'get_unit_digits')
     sale_line = fields.Many2One('sale.line', 'Sale Line', readonly=True,
-            states={
-                'invisible': Not(Equal(Eval('_parent_opportunity.state'),
-                    'converted')),
+        states={
+            'invisible': (Eval('_parent_opportunity', {}).get('state')
+                != 'converted'),
             })
 
     def __init__(self):
@@ -298,7 +348,7 @@ class SaleOpportunityLine(ModelSQL, ModelView):
         self._order.insert(0, ('sequence', 'ASC'))
 
     def on_change_with_unit_digits(self, vals):
-        uom_obj = self.pool.get('product.uom')
+        uom_obj = Pool().get('product.uom')
         if vals.get('unit'):
             uom = uom_obj.browse(vals['unit'])
             return uom.digits
@@ -314,7 +364,7 @@ class SaleOpportunityLine(ModelSQL, ModelView):
         return res
 
     def on_change_product(self, vals):
-        product_obj = self.pool.get('product.product')
+        product_obj = Pool().get('product.product')
 
         if not vals.get('product'):
             return {}
@@ -338,7 +388,7 @@ class SaleOpportunityLine(ModelSQL, ModelView):
         :return: a dictionary with sale line fields as key
             and sale line values as value
         '''
-        sale_line_obj = self.pool.get('sale.line')
+        sale_line_obj = Pool().get('sale.line')
         res = {
             'type': 'line',
             'quantity': line.quantity,
@@ -385,33 +435,35 @@ class SaleOpportunityHistory(ModelSQL, ModelView):
     }, depends=['state'])
     description = fields.Char('Description')
     comment = fields.Text('Comment')
-    lines = fields.Function(fields.One2Many('sale.opportunity.line', None, 'Lines',
-            datetime_field='date'), 'get_lines')
+    lines = fields.Function(fields.One2Many('sale.opportunity.line', None,
+            'Lines', datetime_field='date'), 'get_lines')
     state = fields.Selection(STATES, 'State')
     probability = fields.Integer('Conversion Probability')
     lost_reason = fields.Text('Reason for loss', states={
         'invisible': Not(Equal(Eval('state'), 'lost')),
     }, depends=['state'])
 
-
     def __init__(self):
         super(SaleOpportunityHistory, self).__init__()
         self._order.insert(0, ('date', 'DESC'))
 
     def _table_query_fields(self):
-        opportunity_obj = self.pool.get('sale.opportunity')
+        opportunity_obj = Pool().get('sale.opportunity')
         table = '%s__history' % opportunity_obj._table
         return [
             'MIN("%s".__id) AS id' % table,
             '"%s".id AS opportunity' % table,
-            'MIN(COALESCE("%s".write_date, "%s".create_date)) AS date' % (table, table),
-            'COALESCE("%s".write_uid, "%s".create_uid) AS user' % (table, table),
-        ] + ['"%s"."%s"' % (table, name) for name, field in self._columns.iteritems()
+            ('MIN(COALESCE("%s".write_date, "%s".create_date)) AS date'
+                % (table, table)),
+            ('COALESCE("%s".write_uid, "%s".create_uid) AS user'
+                % (table, table)),
+            ] + ['"%s"."%s"' % (table, name)
+                for name, field in self._columns.iteritems()
                 if name not in ('id', 'opportunity', 'date', 'user')
                 and not hasattr(field, 'set')]
 
     def _table_query_group(self):
-        opportunity_obj = self.pool.get('sale.opportunity')
+        opportunity_obj = Pool().get('sale.opportunity')
         table = '%s__history' % opportunity_obj._table
         return [
             '"%s".id' % table,
@@ -422,7 +474,7 @@ class SaleOpportunityHistory(ModelSQL, ModelView):
                 and not hasattr(field, 'set')]
 
     def table_query(self):
-        opportunity_obj = self.pool.get('sale.opportunity')
+        opportunity_obj = Pool().get('sale.opportunity')
         return (('SELECT ' + \
                 (', '.join(self._table_query_fields())) + \
                 ' FROM "%s__history" '
@@ -431,7 +483,7 @@ class SaleOpportunityHistory(ModelSQL, ModelView):
                 opportunity_obj._table, [])
 
     def get_lines(self, ids, name):
-        line_obj = self.pool.get('sale.opportunity.line')
+        line_obj = Pool().get('sale.opportunity.line')
         histories = self.browse(ids)
         result = {}
         # We will always have only one id per call due to datetime_field
@@ -441,7 +493,6 @@ class SaleOpportunityHistory(ModelSQL, ModelView):
             ])
         return result
 
-
     def read(self, ids, fields_names=None):
         res = super(SaleOpportunityHistory, self).read(ids,
                 fields_names=fields_names)
@@ -449,6 +500,10 @@ class SaleOpportunityHistory(ModelSQL, ModelView):
         # Remove microsecond from timestamp
         for values in res:
             if 'date' in values:
+                if isinstance(values['date'], basestring):
+                    values['date'] = datetime.datetime(
+                        *time.strptime(values['date'],
+                            '%Y-%m-%d %H:%M:%S.%f')[:6])
                 values['date'] = values['date'].replace(microsecond=0)
         return res
 
@@ -486,7 +541,7 @@ class SaleOpportunityEmployee(ModelSQL, ModelView):
         return ['lost']
 
     def table_query(self):
-        opportunity_obj = self.pool.get('sale.opportunity')
+        opportunity_obj = Pool().get('sale.opportunity')
         clause = ' '
         args = [True]
         if Transaction().context.get('start_date'):
@@ -522,7 +577,8 @@ class SaleOpportunityEmployee(ModelSQL, ModelView):
         res = {}
         for record in self.browse(ids):
             if record.number:
-                res[record.id] = float(record.converted) / record.number * 100.0
+                res[record.id] = (float(record.converted)
+                    / record.number * 100.0)
             else:
                 res[record.id] = 0.0
         return res
@@ -552,50 +608,33 @@ class SaleOpportunityEmployee(ModelSQL, ModelView):
 SaleOpportunityEmployee()
 
 
-class OpenSaleOpportunityEmployeeInit(ModelView):
-    'Open Sale Opportunity per Employee Init'
-    _name = 'sale.open_opportunity_employee.init'
+class OpenSaleOpportunityEmployeeStart(ModelView):
+    'Open Sale Opportunity per Employee'
+    _name = 'sale.opportunity_employee.open.start'
     _description = __doc__
     start_date = fields.Date('Start Date')
     end_date = fields.Date('End Date')
 
-OpenSaleOpportunityEmployeeInit()
+OpenSaleOpportunityEmployeeStart()
 
 
 class OpenSaleOpportunityEmployee(Wizard):
     'Open Sale Opportunity per Employee'
-    _name = 'sale.open_opportunity_employee'
-    states = {
-        'init': {
-            'result': {
-                'type': 'form',
-                'object': 'sale.open_opportunity_employee.init',
-                'state': [
-                    ('end', 'Cancel', 'tryton-cancel'),
-                    ('open', 'Open', 'tryton-ok', True),
-                ],
-            },
-        },
-        'open': {
-            'result': {
-                'type': 'action',
-                'action': '_action_open',
-                'state': 'end',
-            },
-        },
-    }
+    _name = 'sale.opportunity_employee.open'
 
-    def _action_open(self, data):
-        model_data_obj = self.pool.get('ir.model.data')
-        act_window_obj = self.pool.get('ir.action.act_window')
-        act_window_id = model_data_obj.get_id('sale_opportunity',
-                'act_opportunity_employee_form')
-        res = act_window_obj.read(act_window_id)
-        res['pyson_context'] = PYSONEncoder().encode({
-            'start_date': data['form']['start_date'],
-            'end_date': data['form']['end_date'],
-            })
-        return res
+    start = StateView('sale.opportunity_employee.open.start',
+        'sale_opportunity.opportunity_employee_open_start_view_form', [
+            Button('Cancel', 'end', 'tryton-cancel'),
+            Button('Open', 'open_', 'tryton-ok', default=True),
+            ])
+    open_ = StateAction('sale_opportunity.act_opportunity_employee_form')
+
+    def do_open_(self, session, action):
+        action['pyson_context'] = PYSONEncoder().encode({
+                'start_date': session.start.start_date,
+                'end_date': session.start.end_date,
+                })
+        return action, {}
 
 OpenSaleOpportunityEmployee()
 
@@ -639,7 +678,7 @@ class SaleOpportunityMonthly(ModelSQL, ModelView):
         return ['lost']
 
     def table_query(self):
-        opportunity_obj = self.pool.get('sale.opportunity')
+        opportunity_obj = Pool().get('sale.opportunity')
         type_id = FIELDS[self.id._type].sql_type(self.id)[0]
         type_year = FIELDS[self.year._type].sql_type(self.year)[0]
         return ('SELECT CAST(id AS ' + type_id + ') AS id, create_uid, '
@@ -675,7 +714,8 @@ class SaleOpportunityMonthly(ModelSQL, ModelView):
         res = {}
         for record in self.browse(ids):
             if record.number:
-                res[record.id] = float(record.converted) / record.number * 100.0
+                res[record.id] = (float(record.converted)
+                    / record.number * 100.0)
             else:
                 res[record.id] = 0.0
         return res
@@ -750,7 +790,7 @@ class SaleOpportunityEmployeeMonthly(ModelSQL, ModelView):
         return ['lost']
 
     def table_query(self):
-        opportunity_obj = self.pool.get('sale.opportunity')
+        opportunity_obj = Pool().get('sale.opportunity')
         type_id = FIELDS[self.id._type].sql_type(self.id)[0]
         type_year = FIELDS[self.year._type].sql_type(self.year)[0]
         return ('SELECT CAST(id AS ' + type_id + ') AS id, create_uid, '
@@ -789,7 +829,8 @@ class SaleOpportunityEmployeeMonthly(ModelSQL, ModelView):
         res = {}
         for record in self.browse(ids):
             if record.number:
-                res[record.id] = float(record.converted) / record.number * 100.0
+                res[record.id] = (float(record.converted)
+                    / record.number * 100.0)
             else:
                 res[record.id] = 0.0
         return res
